#include <libmp/all.h>
#include <stdio.h>

#include "../util.h"
#include "../cmd.h"
#include "../log.h"

bool cmd_install_callback(lm_ctx_t *ctx, lm_pkg_t *pkg, char *file, size_t current, size_t total, void *data){
  bar(current, total); 
  return true;
}

bool cmd_download_callback(lm_ctx_t *ctx, lm_pkg_t *pkg, bool is_archive, size_t current, size_t total, void *data){
  if(!is_archive)
    bar(current, total);
  return true;
}

bool cmd_install(lm_ctx_t *ctx, config_t *config, args_t *args){
  if(args_get_bool(args, "help")){
    info(_("Listing options for the install command:"));
    printf(_("    "FG_BOLD"--continue"FG_RESET":\tcontinue even if package is not found\n"));
    printf(_("    "FG_BOLD"--yes"FG_RESET":\tdon't ask for confirmation\n"));
    return true;
  }
  
  lm_ctx_resolve_list_t *list = NULL;
  bool ret = false, r = false;
  ssize_t current = 0, size = 0;
  char ssize[LONGSTR_MAX+3];
  lm_pkg_t *pkg = NULL;
  char *name = NULL;

  if(lm_ctx_sync(ctx, false, NULL, NULL) <= 0){
    error(_("There are no avaliable pools, have you synced yet?"));
    goto end;
  }

  for(int i = 0; i < args->count; i++){
    if(NULL != args->list[i].name)
      continue;

    if(eq((name = args->list[i].value), "install"))
      continue;

    // find the package
    if((pkg = lm_ctx_pool_find(ctx, name, NULL)) == NULL){
      error(_("Failed to find "FG_BOLD"%s"FG_RESET": %s"), name, lm_strerror());
      if(args_get_bool(args, "continue"))
        continue;
      else
        goto end;
    }

    if(lm_ctx_database_is_installed(ctx, pkg, true)){
      error(_("Package "FG_BOLD"%s"FG_RESET" ("FG_BLUE"%s"FG_RESET") is already installed"), name, pkg->version);
      goto end;
    }
  
    // resolve the package and depends
    if((list = lm_ctx_resolve(ctx, pkg, list)) == NULL){
      error(_("Failed to resolve "FG_BOLD"%s"FG_RESET": %s"), name, lm_strerror());
      goto end;
    }
  }

  if(NULL == list){
    error(_("Nothing to do (no packages to install)"));
    goto end;
  }
  
  while((pkg = lm_ctx_resolve_next(list)) != NULL)
    size += pkg->size;
  size_to_human(ssize, size);

  if(list->count == 1)
    info(_("Following %d package will be "FG_BOLD"INSTALLED:"FG_RESET), list->count);
  else
    info(_("Following %d packages will be "FG_BOLD"INSTALLED:"FG_RESET), list->count);
  
  // list all the packages

  info(_("Total of "FG_BOLD"%s"FG_RESET" disk space will be used"), ssize);
  if(!args_get_bool(args, "yes") && !yesno(_("Continue?"))){
    error(_("Operation cancelled"));
    goto end;
  }

  // download resolved packages
  while((pkg = lm_ctx_resolve_next(list)) != NULL){
    info(_("(%d/%d) Downloading "FG_BOLD"%s"FG_RESET" ("FG_BOLD FG_BLUE"%s"FG_RESET")"), ++current, list->count, pkg->name, pkg->version);

    r = lm_ctx_download(ctx, pkg, cmd_download_callback, NULL);
    bar_free();

    if(!r){
      error(_("Failed to download "FG_BOLD"%s"FG_RESET": %s"), pkg->name, lm_strerror());
      goto end;
    }
  }

  current = 0;

  // install resolved packages
  while((pkg = lm_ctx_resolve_next(list)) != NULL){
    info(_("(%d/%d) Installing "FG_BOLD"%s"FG_RESET" ("FG_BOLD FG_BLUE"%s"FG_RESET")"), ++current, list->count, pkg->name, pkg->version);

    r = lm_ctx_install(ctx, pkg, cmd_install_callback, NULL);
    bar_free();

    if(!r){
      error(_("Failed to install "FG_BOLD"%s"FG_RESET": %s"), pkg->name, lm_strerror());
      goto end;
    }
  }

  if(current == list->count){
    success(list->count == 1 ? _("Installed %d package") : _("Installed all of the %d packages"), list->count);
    ret = true;
  }

end:
  bar_free();
  lm_ctx_resolve_free(list);
  return ret;
}
