#include <libmp/all.h>
#include <libmp/ctx.h>
#include <libmp/error.h>
#include <stdio.h>

#include "../cmd.h"
#include "../log.h"

struct cmd_update_data {
  ssize_t total;
  ssize_t current;
};

bool cmd_update_callback(lm_ctx_t *ctx, lm_pkg_t *pkg, lm_ctx_update_state_t state, char *file, size_t current, size_t total, void *data){
  struct cmd_update_data *cbdata = data;
  
  switch (state) {
  case UPDATE_REMOVE:
    if(current == 0)
      info(_("(%d/%d) Removing "FG_BOLD"%s"FG_RESET" ("FG_BOLD FG_BLUE"%s"FG_RESET")"), cbdata->current, cbdata->total, pkg->name, pkg->version);
    bar(current, total);
    break;

  case UPDATE_INSTALL:
    if(current == 0){
      bar_free();
      info(_("(%d/%d) Installing "FG_BOLD"%s"FG_RESET" ("FG_BOLD FG_BLUE"%s"FG_RESET")"), cbdata->current, cbdata->total, pkg->name, pkg->version);
    }
    bar(current, total);
    break;
  }

  return true;
}

bool cmd_update(lm_ctx_t *ctx, config_t *config, args_t *args){
  if(args_get_bool(args, "help")){
    info(_("Listing options for the update command:"));
    printf(_("    "FG_BOLD"--yes"FG_RESET":\tdon't ask for confirmation\n"));
    return true;
  }

  lm_ctx_update_list_t *list = NULL;
  struct cmd_update_data cbdata;
  bool ret = false, r = false;
  lm_pkg_t *pkg = NULL;
  ssize_t current = 0;

  if((list = lm_ctx_update_list(ctx)) == NULL){
    error(_("Failed to get the package update list: %s"), lm_strerror());
    goto end;
  }

  if(list->count == 0){
    error(_("All packages are up-to-date"));
    goto end;
  }

  if(list->count == 1)
    info(_("Following %d package will be "FG_BOLD"UPDATED:"FG_RESET), list->count);
  else
    info(_("Following %d packages will be "FG_BOLD"UPDATED:"FG_RESET), list->count);
  
  // list all the packages

  if(!args_get_bool(args, "yes") && !yesno(_("Continue?"))){
    error(_("Operation cancelled"));
    goto end;
  }

  while((pkg = lm_ctx_update_list_next(list)) != NULL){
    cbdata.current = ++current;
    cbdata.total = list->count;

    r = lm_ctx_update(ctx, pkg, cmd_update_callback, &cbdata);
    bar_free();

    if(!r){
      error(_("Failed to update "FG_BOLD"%s"FG_RESET": %s"), pkg->name, lm_strerror());
      goto end;
    }
  }

  ret = true;
end:
  lm_ctx_update_list_free(list);
  return ret;
}
